//------------------------------------------------------------------- CDFQEarnings.do
// This code produces Figure 4 (Article),
// that is, it draws the empirical CDF of rescaled (by the FPL) quarterly earnings in the JF (aka Treatment) and Control samples.
// 
// This code is embedded into a program to make the output (written to STATA Results' window) more readable.
//
// This code has four parts:
// 	PART 1: sets parameters/ preferences 
//  PART 2: bring in policy parameters
// 	PART 3: works on the data in entry to prepare it for PART 3-6
// 	PART 4: compute the KS and SD tests
//	PART 5: compute the p-values of the KS and SD tests, by boostrapping
//  PART 6: draw CDFS
//	PART 7: send code for execution 
//
// This code take as inputs:
// 	1. pre-generated MDRC data
// 	2. pre-generated data sets of welfare policy rules 
// 
// This code outputs:
// 	1. .pdf files with the CDFs 
// 
// Last Revision: 08/30/2015
//-------------------------------------------------------------------------------------------------------------------------------
args jf_dtain rules_dtain


	
	
	
	
capture program drop CDFQEarnings
capture program define CDFQEarnings
args jf_dtain rules_dtain



	//------------------------------------------------ PART 1  ------------------------------------------------------
	// Choose what you want to do by setting a few local variables.
	//---------------------------------------------------------------------------------------------------------------

		//-----------------------------------------------------------------------
		// Pick the tasks that you want to run by toggling on and off three indicators:
		//	ind_CDFs_draw 		= 1 	means that you want to graph the CDFs, 0 otherwise
		//	ind_CDFs_KStest 	= 1 	means that you want to compute the KS test, 0 otherwise
		//	ind_CDFs_KStest_bs	= 1 	means that you want to bootstrap the SE of the KS test, 0 otherwise
		//-----------------------------------------------------------------------
		local ind_CDFs_draw 		= 1   
		local ind_CDFs_KStest 		= 1
		local ind_CDFs_KStest_bs 	= 1 // warning: this task can be lenghty

		//-----------------------------------------------------------------------
		// Pick the option concerning how to determine AU size (hence the 
		// FPL value to use to define the earning ranges for each case).
		// The options are nextsizeup 
		//	- nextsizeup: uses FPL for AU Size + 1
		//-----------------------------------------------------------------------
		local whichFPL  "next size up" 	
		//-----------------------------------------------------------------------
		// Pick which postRA quarters you want to use (among those in the reshaped data)
		//-----------------------------------------------------------------------
		local fq = 1
		local lq = 7
		//-----------------------------------------------------------------------
		// Pick the number of bootstraps for the computation of the SE.
		//-----------------------------------------------------------------------
		local nboots = 1000
		//----------------------------------------------------------------------- 
		// Pick the list of variables used in the pscore logistic regression.
		// Because {anyernpq7,ernpq7} are degenerate in g0, {anyernpq7} are degenerate in g1 
		// a small adjustment is needed.
		//-----------------------------------------------------------------------
		global pscorevars "ernpq8 ernpq6-ernpq1 adcpq7-adcpq1 fstpq7-fstpq1 anyernpq1-anyernpq6 anyernpq8 anyadcpq1-anyadcpq7 anyfstpq1-anyfstpq7 "
		global pscorevars "$pscorevars yremp prevafdc white black hisp marnvr marapt agelt25 age2534 "
		global pscorevars "$pscorevars nohsged hsged kidctgt2 applcant"
		global pscorevars "$pscorevars misshs misskidctgt2 missmar"
		global pscorevars_g0 "$pscorevars"
		global pscorevars_g1 "$pscorevars ernpq7"
		global pscorevars "$pscorevars ernpq7 anyernpq7" 
		//-----------------------------------------------------------------------
		// Assign informative names to the files produced below.
		//-----------------------------------------------------------------------
		local whatwedo "Figure4"	
		local log_graphscdf "$oJF/`whatwedo'.log"
		//-----------------------------------------------------------------------
		// Load programs TwoSampleKSTest and TwoSampleKSTest_bs, called below.
		//-----------------------------------------------------------------------
		qui do CDFQEarnings_programs

		
		
		
		
		
		
		
		
	//------------------------------------------------ PART 2  ------------------------------------------------------
	// Bring in Policy Parameters and save them into a data set that is ready to be merged in.
	//---------------------------------------------------------------------------------------------------------------
		qui use "`rules_dtain'", clear 
		keep  F3 year size
		qui rename F3 F3_
		qui reshape wide  F3, i(year) j(size)
		sort year 
		qui save "$ddJF/JF_ADFCrules_temp.dta", replace
			
		
		
		
		
		
		
		


	//------------------------------------------------ PART 3  ------------------------------------------------------
	// Preliminaries needed to run the tasks identified by indicator variables
	// {ind_CDFs_draw, ind_CDFs_KStest, ind_CDFs_KStest_bs}.
	//----------------------------------------------------------------------------------------------------------------

		//----------------------------------------------------------------------
		// Use data and reshape to long format.
		//----------------------------------------------------------------------
		qui use ernpq8-ernpq1 ernq0-ernq7 afdconq1-afdconq7 id pscorewt e ///
				afdcon quarter month applcant kidcount year earnq $pscorevars using "`jf_dtain'" if ///
				quarter>=`fq'&quarter<=`lq', clear		
	
		//----------------------------------------------------------------------
		// Apply kidcount-based sample exclusions and generate the variable for the 
		// AU size, it is needed in order to associate the correct FPL.
		//----------------------------------------------------------------------
		qui drop if kidcount==.
		qui gen size = kidcount +1 if kidcount==1|kidcount==2|kidcount==3
		qui replace size = 2 if kidcount==0			
		//----------------------------------------------------------------------
		// Merge in policy parameters
		//----------------------------------------------------------------------
		sort year
		qui merge m:1 year using  "$ddJF/JF_ADFCrules_temp.dta"
		qui drop if _m~=3 // drop years for which you do not have the rules
		drop _m		
		//----------------------------------------------------------------------
		// Generate the rescaled earnings variable based on the specifications selected above
		//----------------------------------------------------------------------
		qui gen earnq_resc = .
		forvalues y=1996(1)1998 {
			forvalues s=1(1)5 {
				//-----------------------------------------
				// this is the size to use for the FPL, = 
				// `s' means exactly the size in the data, 
				// `s'+1 means the next size up etc.
				//-----------------------------------------
				local ss = `s' +1
				qui sum F3_`ss' if year==`y'
				qui replace earnq_resc = earnq/(3*r(min))  if size==`s'&year==`y' 
			}
		}
		//------------------------------------------------
		// As earning variable use 100$-rounded quarterly earnings rescaled by size-specific FPL
		//------------------------------------------------
		qui gen earnings = earnq_resc  	// 

		//-------------------------------------------------------------------
		// Create vars that identify the preRA earning subgroups (see kt_JF_programs_AER.do)
		//------------------------------------------------------------------- 
		create_earning_subgroup 
		//-------------------------------------------------------------------
		// Reshape wide in order to get (below) the propensity score weights.
		//
		// Save the data because it will be used to bootstrap SE of the KS test (if that is what you want to do).
		// NOTE: we save the data here b/c we have to recompute p-weights on each bootstrapped dataset.
		//-------------------------------------------------------------------
		qui gen nn = .
		qui bys id quarter: replace nn = _n
		qui drop if nn~=1 // wlog we drop 2 of the 3 months in the quarter
		keep id e quarter applcant  earning_subgroup earnings  size kidcount $pscorevars
		qui reshape wide earnings@, i(id) j(quarter)
		qui save "$ddJF/CDFQEarnings1.dta", replace 

		//-------------------------------------------------------------------
		// Run pscore logit, predict propensity score (pscore), generate IPW (pscorewt),
		// and reshape long ( eeach sample unit has as many rows as there are quarters
		// included in the analysis)
		//-------------------------------------------------------------------
		capture drop pscorewt
		qui logit e $pscorevars
		qui predict double pscore                                      
		qui gen double pscorewt = e/pscore + (1-e)/(1-pscore)	
		drop pscore
		//------------------------------------
		qui logit e $pscorevars if kidcount==1
		qui predict double pscore                                      
		qui gen double pscorewt_k1 = e/pscore + (1-e)/(1-pscore)	 if kidcount==1
		drop pscore
		//------------------------------------
		qui logit e $pscorevars if kidcount==2
		qui predict double pscore                                      
		qui gen double pscorewt_k2 = e/pscore + (1-e)/(1-pscore)	 if kidcount==2		
		drop pscore
		//------------------------------------
		foreach g of numlist  0 1 {
			qui logit e ${pscorevars_g`g'} if earning_subgroup == `g'
			qui predict double pscore                                      
			qui gen double pscorewt_g`g' = e/pscore + (1-e)/(1-pscore)	if earning_subgroup == `g'
			drop pscore
		}
		//------------------------------------ 
		keep id e applcant size kidcount earning_subgroup earnings* pscorewt*
		qui reshape long earnings@, i(id) j(qtr)
		qui gen quantile = _n if _n<100
		qui save "$ddJF/CDFQEarnings2.dta", replace

	
	capture log close
	log using "$oJF/`whatwedo'.log", replace		
		
		
		
		
		
		
		
	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Compute the KS and DS tests.
	//
	//  Call the program (TwoSampleKSTest) that computes the two CDFs and their difference (on the data and/or its subgroups)  
	//  and stores the two CDFs and their difference in a .dta file. It also writes to screen the value of 
	//  the KS statistics and of the SD statistics and stores them into the matrix TS.
	//  Notes: 
	//	a.	We use rescaled (by FPL) earning data when computing K-S Test
	//	b.	This part of the code is called even if we just want to compute p-value for the KS test  
	// 		  by means of boostrapbecause we need the  original CDFs  to carry out that task.
	//----------------------------------------------------------------------------------------------------------------		
	if `ind_CDFs_KStest'==1 | `ind_CDFs_KStest_bs'==1  {
		//---------------------------------------------------
		// The rows of matrix TS accomodate the samples, 
		// the cols accommodate the tests which are KS and SD
		//---------------------------------------------------
		matrix define TS = J(3,2,0) 
		matrix rownames TS = "all" "g=0" "g=1"
		matrix colnames TS = "KS Test" "DS Test"
		local c = 1
		TwoSampleKSTest   pscorewt    "all"  `c' "$ddJF/CDFQEarnings2.dta" "$ddJF/EmpiricalReweightedCDFs_all.dta"
		foreach g of numlist  0 1 {
			local c = `c'+1
			TwoSampleKSTest   pscorewt_g`g' `g'  `c' "$ddJF/CDFQEarnings2.dta" "$ddJF/EmpiricalReweightedCDFs_g`g'.dta"
		}
		//---------------------------------------------------
		di ""
		di ""
		di ""
		di "KS (SD) test for:"
		di "	- all cases"
		di "	- cases with zero earnings in 7th quarter preRA (g=0)"
		di "	- cases with positive earnings in 7th quarter preRA (g=1)"
		matrix list TS, noheader format(%6.3f)
	}
		



		
		
		
		
	//------------------------------------------------ PART 5  ------------------------------------------------------
	// Compute the p-values of the KS and DS tests, by bootstrapping
	//----------------------------------------------------------------------------------------------------------------			

		//------------------------------------------------------------------------
		// Draw random samples (if we want to) and for each compute the KS and SD statistics, store.
		//------------------------------------------------------------------------
		if `ind_CDFs_KStest_bs'==1  { 
			set seed 112233
			//-----------------------------------------------------------------------
			matrix define BS = J(`nboots',2,0) 
			matrix colnames BS = "KS Test" "DS Test"
			forvalues r=1(1)`nboots' { 
				TwoSampleKSTest_bs  pscorewt "all"  `r' "$ddJF/CDFQEarnings1.dta" "$ddJF/EmpiricalReweightedCDFs_all.dta"  
			}
			//-----------------------------------------------------------------------
			qui svmat BS
			keep BS1 BS2
			qui keep if _n<=`nboots'
			notes : "Sample all. Earnings rescaled based on FPL. Use to obtain the bootstrapped distribution under Ho of the K-S 2-sample statistics."
			label variable BS1 "K-S 2-sample statistics"
			label variable BS2 "Stochastic Dominance 2-sample statistics"
			qui save "$ddJF/`whatwedo'_all.dta", replace
			//-----------------------------------------------------------------------
			foreach g of numlist  0 1 {
				matrix define BS = J(`nboots',2,0)
				matrix colnames BS = "KS Test" "DS Test"
				forvalues r=1(1)`nboots' { 
					TwoSampleKSTest_bs pscorewt `g' `r' "$ddJF/CDFQEarnings1.dta" "$ddJF/EmpiricalReweightedCDFs_g`g'.dta" 
				}
				qui svmat BS
				keep BS1 BS2
				qui keep if _n<=`nboots'
				notes : "Sample `g'. Earnings rescaled based on FPL. Use to obtain the bootstrapped distribution under Ho of the K-S 2-sample statistics."
				label variable BS1 "K-S 2-sample statistics"
				label variable BS2 "Stochastic Dominance 2-sample statistics"
				qui save "$ddJF/`whatwedo'_g`g'.dta", replace
			} // loop over earning subgroups
		} // If we want to redraw the bs samples
		
		//------------------------------------------------------------------------
		// Use just drawn random samples or use those in existence to compute
		// the p-value of the KS test and of the DS test.
		//------------------------------------------------------------------------
		if `ind_CDFs_KStest'==1|`ind_CDFs_KStest_bs'==1 { 
			matrix define P = J(3,2,0)
			matrix colnames P = "p-value" "p-value"
			local ks = TS[1,1]
			local sd = TS[1,2]
			qui use "$ddJF/`whatwedo'_all.dta", clear
			qui gen alpha1 = (BS1>`ks')
			qui gen alpha2 = (BS2>`sd')
			qui sum alpha1
			matrix define P[1,1] = r(mean)
			*di "In sample all the bootstrapped p-value of the KS test has value = ", r(mean)
			qui sum alpha2
			matrix define P[1,2] = r(mean)
			*di "In sample all the bootstrapped p-value of the SD test has value = ", r(mean)
			local r = 1
			foreach g of numlist  0 1 {
				local r = `r'+1
				local ks = TS[`r',1]
				local sd = TS[`r',2]
				qui use "$ddJF/`whatwedo'_g`g'.dta", clear
				qui gen alpha1 = (BS1>`ks')
				qui gen alpha2 = (BS2>`sd')
				qui sum alpha1
				matrix define P[`r',1] = r(mean)
				*di "In sample g`g' the bootstrapped p-value of the KS test has value = ", r(mean)
				qui sum alpha2
				matrix define P[`r',2] = r(mean)
				*di "In sample g`g' the bootstrapped p-value of the SD test has value = ", r(mean)
			} // loop over earning subgroups
			//---------------------------------------------------
			matrix define TS2 = (TS[1...,1],P[1...,1],TS[1...,2],P[1...,2])
			di ""
			di ""
			di ""
			di "KS (SD) test and bootstrapped p-value of the KS (SD) test for:"
			di "	- all cases"
			di "	- cases with zero earnings in 7th quarter preRA"
			di "	- cases with positive earnings in 7th quarter preRA"
			matrix list TS2, noheader format(%6.3f)
		}
	log close
	
	
	
	
		
		
		
	//------------------------------------------------ PART 6  ------------------------------------------------------
	// Graphing the CDFs
	//----------------------------------------------------------------------------------------------------------------	
	if `ind_CDFs_draw'==1 {
		qui use "$ddJF/CDFQEarnings2.dta", clear		
		//--------------------------------------------------------
		// CDF of Quarterly Earnings by Treatment, Q1-Q7
		// Sample: All units with non missing kidcount at RA.
		// Uses cumul and inverse propensity score weights. Earnings equal to 0 and above 2.5 times the FPL.
		// are not included in the graph. The vertical line is drawn at 1 where earnings equal 3*FPL.
		//--------------------------------------------------------
		capture drop cumvar*
		qui cumul earnings  if qtr<=7 [aw=pscorewt], gen(cumvar) by(e)	equal		
		sort e cumvar
		//----------------------------------------------- 
		graph twoway line  cumvar earnings if e==0&earnings<2.5&earnings>0 || ///
					 line  cumvar earnings if e==1&earnings<2.5&earnings>0, lpattern("--######")  ///
					 legend( lab(1 "AFDC") lab(2 "JF")) graphregion(color(white)) bgcolor(white) ///
					 xline(1, lpattern(dash) lcolor(black))  ///
					 ytitle("") xtitle("Quarterly Earnings as a fraction of 3*FPL.", size(small)) ///
					 graphregion(color(white)) bgcolor(white) ///
					 legend( ///
						region(lcolor(white)) ///
						symxsize(12) ///
						cols(2) ///
					) name(`whatwedo'_all, replace)
		graph export "${gJF}/`whatwedo'_all.pdf", logo(off) orientation(landscape) mag(100) replace  
		capture drop cumvar	
		//-----------------------------------------------------------------------------------------------------------
		// CDF of Quarterly Earnings by Treatment, Q1-Q7.
		// Sample: `sample_g`g'' with non missing kidcount at RA.
		// Uses cumul and inverse propensity score weights. Earnings equal to 0 and above 7.5 times the FPL
		// are not included in the graph. The vertical line is drawn at 1 where earnings equal 3*FPL.
		//-----------------------------------------------------------------------------------------------------------
		local sample_g0 "Zero preRA Earnings"
		local sample_g1 "Positive preRA Earnings"
		foreach g of numlist  0 1 {
			cumul earnings  if qtr<=7&earning_subgroup==`g' [aw=pscorewt_g`g'], gen(cumvar) by(e) equal
			sort e cumvar
			graph twoway line  cumvar earnings if e==0&earnings<2.5&earnings>0 || ///
						 line  cumvar earnings if e==1&earnings<2.5&earnings>0, lpattern("--######") ///
						 legend( lab(1 "AFDC") lab(2 "JF")) graphregion(color(white)) bgcolor(white) ///
						 xline(1, lpattern(dash) lcolor(black))  ///
						 ytitle("") xtitle("Quarterly Earnings as a fraction of 3*FPL.", size(small)) ///
						 graphregion(color(white)) bgcolor(white) ///
						 legend( ///
							region(lcolor(white)) ///
							symxsize(12) ///
							cols(2) ///
						) name(`whatwedo'_g`g', replace)	
			graph export "${gJF}/`whatwedo'_g`g'.pdf", logo(off) orientation(landscape) mag(100) replace 
			capture drop cumvar	
		}
	}	
	
	//---------------------------------------------------------------
	erase "$ddJF/CDFQEarnings1.dta"
	erase "$ddJF/CDFQEarnings2.dta"
	erase "$ddJF/EmpiricalReweightedCDFs_all.dta"
	erase "$ddJF/EmpiricalReweightedCDFs_g0.dta"
	erase "$ddJF/EmpiricalReweightedCDFs_g1.dta"
	erase "$ddJF/`whatwedo'_all.dta"
	erase "$ddJF/`whatwedo'_g0.dta"
	erase "$ddJF/`whatwedo'_g1.dta"
	erase "$ddJF/JF_ADFCrules_temp.dta"	

end // encapsulating program





	//------------------------------------------------ PART 7  ------------------------------------------------------
	// Execution
	//---------------------------------------------------------------------------------------------------------------
	CDFQEarnings "`jf_dtain'" "`rules_dtain'"
	


	
	
	
	
	
	
	
	
	
