//------------------------------------------------------------------- CDFQEarnings_programs
// This code contains two programs called by CDFQEarning.do
//
// Last Revision: 09/03/2015
//-------------------------------------------------------------------------------------------------------------------------------

	
capture program drop TwoSampleKSTest
capture program define TwoSampleKSTest
	args wv g r dtain dtaout 
	//-------------------------------- Inputs & Outputs
	// wv 			weight variable
	// g 			earnings-group
	// r			row in the matrix storing the tests
	// dtain		file .dta in input
	// dtaout		file .dta storing CDFs of earnings and their differences
	//--------------------------------
	qui use "`dtain'", clear
	local suffix "`g'"
	if "`g'"~="all" {
		qui keep if earning_subgroup==`g'
		local suffix "g`g'"
	}	
	//---------------------------------------------------------------------
	// Declare some temporary data files
	//---------------------------------------------------------------------
	tempfile cum
	tempfile cumAFDC

	//---------------------------------------------------------------------
	// We use STATA command "cumul, equal" and the rescaled earnings to obtain
	// the CDG of rescaled earnings in the treatment and in the control samples
	//---------------------------------------------------------------------
	capture drop cumvar*
	qui cumul earnings   [aw=`wv'], gen(cumvar) by(e) equal
	keep earnings cumvar e
	// by contracting, the data contains for each sample (control, experimental) the CDF
	qui contract _all 
	drop _freq
	// check: confirms that cumvar was the same for every earning level that was the same (within treatment groups)
	qui gen n = .
	qui bys e earnings: replace n = _N
	capture assert n==1
	if _rc~=0 {
		di "We are processing the data for sample `suffix'  in order to compute the two-sample K-S test"
		di "Contrary to expectations there are multiple values of cumvar attached to the same level or rescaled earnings"
		sort e earnings
		list e earnings cumvar if n>1, ab(30)
		pause on
		pause
	}
	keep earnings cumvar e
	qui save "`cum'", replace
	//------------------------------- 
	// We bring the CDFs side by side so that below we can take differences
	//------------------------------- 
	qui use "`cum'" if e==0, replace
	drop e
	rename cumvar cumvarAFDC
	label variable cumvarAFDC "CDF of rescaled earnings (relative to FPL) in control sample"
	sort earnings
	qui save "`cumAFDC'", replace
	qui use "`cum'" if e==1, replace
	drop e
	rename cumvar cumvarJF
	label variable cumvarJF "CDF of rescaled earnings (relative to FPL) in JF (aka treatment) sample"
	sort earnings
	qui merge 1:1 earnings using "`cumAFDC'"	
	drop _merge			
	//-----------------------------------
	// We replace missing with the lagged value since the CDF 
	// does not go up at points not in the support (that below we can take differences)
	//------------------------------- 
	foreach var of varlist cumvar* {
		forvalues i=1(1)30 {
			sort earnings
			qui replace `var' = `var'[_n-1] if `var'[_n]==.&`var'[_n-1]~=.
		}
		capture assert `var'~=. // after the replacements from below
		if _rc~=0 {
			di "We are processing the data for sample `suffix'  in order to compute the two-sample K-S test"
			di "Contrary to expectations there are still missing in the CDF of some rescaled earnings after the merge"
			di "of the AFDC-CDF and JF-CDF."
			list e earnings `var' if `var'==., ab(30)
			pause on
			pause
		}
	}
	//----------------------------------
	// We compute the KS statistics  as the max of the 
	// absolute difference b/w the two CDFs
	//----------------------------------
	qui gen cumdiff = abs(cumvarJF-cumvarAFDC)
	qui sum cumdiff
	*di "In sample `suffix', the two-sample K-S test has value = " r(max) 
	matrix define TS[`r',1] = r(max) 
	//----------------------------------
	// We compute the Stochastic-Dominance statistics 
	// as the max of the  difference b/w CDF-JF and CDF-AFDC 
	//----------------------------------			
	qui gen cumdiff2 = cumvarJF-cumvarAFDC
	qui sum cumdiff2
	*di "In sample `suffix', the two-sample Stochastic Dominance test has value = " r(max) 		
	matrix define TS[`r',2] = r(max) 
	//-----------------------------------
	notes : "Sample `suffix'. CDFs of control and treatment groups and absolute differences."
	label variable earnings "Rescaled (relative to FPL) quarterly earnings"
	label variable cumdiff  "Absolute Difference b/w CDFs of earnings, Treatment - Control"
	label variable cumdiff2 "Difference b/w CDFs of earnings, Treatment - Control"
	sort  earnings
	qui save "`dtaout'", replace
end














//---------------------------------------------------------------------------------------------------
capture program drop TwoSampleKSTest_bs
capture program define TwoSampleKSTest_bs
	args wv g r dtain origcdfdta 
	
	//--------------------------------	
	// wv 			weight variable
	// g 			earnings-group
	// r 			bootstrap counter
	// dtain		file .dta in input
	// origcdfdta	file .dta in input containing data-based CDFs
	//--------------------------------

	
	//--------------------------------
	qui use "`dtain'", clear
	local suffix "all"
	if "`g'"~="all" {
		local suffix "g`g'"
		qui keep if earning_subgroup==`g'
	}
	if `r'==1 {
		di ""
		di ""
		di ""
		di "=========================================== Bootstrapping with replacement (sample `suffix')"
		di "- working on bootstrapped sample `r' (boostrapping takes time ...)"
	}
	forvalues i=0(1)10 {
		if `r'==`i'*100 {
			di "- working on bootstrapped sample `r' "
		}
	}


	//---------------------------------------------------------------------
	// Here we use cumul, equal and the rescaled earnings (by FPL)
	//---------------------------------------------------------------------
		tempfile cum
		tempfile cumAFDC
		
		
		//---------------------------------------------------
		// Draw a random sample from existing data
		// Run pscore logit, predict propensity score (pscore), and generate IPW (pscorewt)
		// Reshape long (over quarters)
		//---------------------------------------------------
		rename id pufid
		qui bsample, cluster(pufid) idcluster(id)		

		capture drop `wv'
		qui logit e $pscorevars, asis
		qui predict double pscore                                      
		gen double `wv' = e/pscore + (1-e)/(1-pscore)	
		capture assert `wv'~=.
		if _rc~=0 {
			list e $pscorevars pscore if `wv'==., ab(30)
			pause on
			pause
		}
		drop pscore
		
		
		keep id e earnings* `wv'
		qui reshape long earnings@, i(id) j(qtr)
		


		//----------------------------------------------------
		// Computed the CDFs of rescaled earnings for each treatment sample
		//----------------------------------------------------
		capture drop cumvar*
		qui cumul earnings  [aw=`wv'], gen(cumvar) by(e)	equal
		keep earnings cumvar e
		qui contract _all
				
		qui save "`cum'", replace
		qui use "`cum'" if e==0, replace
		drop e
		rename cumvar cumvarAFDC
		sort earnings
		qui save "`cumAFDC'", replace
		qui use "`cum'" if e==1, replace
		drop e
		rename cumvar cumvarJF
		sort earnings
		qui merge 1:1 earnings using "`cumAFDC'"
		// now we replace missing with the lagged value since the CDF does not go up at points not in the support
		foreach var of varlist cumvar* {
			forvalues i=1(1)20 {
				sort earnings
				qui replace `var' = `var'[_n-1] if `var'[_n]==.&`var'[_n-1]~=.
			}
			assert `var'~=. // after the replacements from below
		}
		rename _merge newm
		//----------------------------------------------------
		// Merge in the values of the CDFs from actual data
		//----------------------------------------------------
		rename cumvarJF cumvarJF_bs
		rename cumvarAFDC cumvarAFDC_bs
		sort earnings
		qui merge 1:1 earnings using "`origcdfdta'"	
		
		//----------------------------------------------------
		// Handle situations when in the bs-data CDFs some points of support are missing
		// but they are present in the actual-data CDFs. We backfill.
		//----------------------------------------------------
		foreach var of varlist cumvar*_bs {
			forvalues i=1(1)20 {
				sort earnings
				qui replace `var' = `var'[_n-1] if `var'[_n]==.&`var'[_n-1]~=.
			}
			// assert that after the replacements from below the bs-CFs have no missing
			assert `var'~=. 
		}		

		//----------------------------------------------------
		// We compute the KS statistics
		//----------------------------------------------------
		capture drop cumdiff
		qui gen cumdiff = abs( (cumvarJF-cumvarAFDC)-(cumvarJF_bs-cumvarAFDC_bs) )
		qui sum cumdiff
		matrix define BS[`r',1] = r(max) 
		//----------------------------------------------------
		// We compute the Stochastic Dominance statistics
		//----------------------------------------------------
		capture drop cumdiff2
		qui gen cumdiff2 = (cumvarJF-cumvarAFDC) -(cumvarJF_bs-cumvarAFDC_bs)
		qui sum cumdiff2
		matrix define BS[`r',2] = r(max) 
end








