//------------------------------------------------------------------- MeanPostRAOutcomes.do
// This code produces Table A2 (Online Appendix),
// that is, it computes the mean of several outcomes during the postRA period from quarter 1 to 7.
// 
// This code is embedded into a program to make the output (written to STATA Results' window) more readable.
//
// This code has four parts:
// 	PART 1: sets parameters/ preferences 
//  PART 2: bring in policy parameters
//  PART 3: construct intermediate data set used in subsequent parts of the code
// 	PART 4: compute mean impacts
// 	PART 5: Compute SEs by boostrapping
//	PART 6: send code for execution
//
// This code take as inputs:
// 	1. pre-generated MDRC data
// 	2. pre-generated data sets of welfare policy rules 
// 
// This code outputs:
// 	1. .log files with mean impacts and their boostrapped SEs
// 
// Last Revision: 08/31/2015
//-------------------------------------------------------------------------------------------------------------------------------
args jf_dtain rules_dtain








capture program drop MeanPostRAOutcomes
capture program define MeanPostRAOutcomes
args jf_dtain rules_dtain

	//------------------------------------------------ PART 1  ------------------------------------------------------
	// Choose what you want to do by setting a few local variables.
	//---------------------------------------------------------------------------------------------------------------
	
	
		//-----------------------------------------------------------------------
		// Pick the tasks that you want to run by toggling on and off three indicators:
		// 		ind_distri = 1 if want to get mean sample characteristics in the data, otherwise 0
		// 		ind_bootst = 1 if want to get SEs for the mean sample characteristics, otherwise 0
		//
		// Pick the number of bootstraps for the computation of the SE.
		//-----------------------------------------------------------------------
		local ind_descst = 1
		local ind_bootst = 1 // warning: this task can be lenghty
		local nboots = 1000
		//-----------------------------------------------------------------------
		// Select which postRA quarters you want to use (among those in the reshaped data)
		//-----------------------------------------------------------------------
		local fq = 1 
		local lq = 7 

		//-----------------------------------------------------------------------
		// Assign informative names to the files produced below.
		//-----------------------------------------------------------------------
		local whatwedo "TableA2"
		local logfileout		"$oJF/`whatwedo'.log"		// for keeping track
		//-----------------------------------------------------------------------
		// Declare local programs
		//-----------------------------------------------------------------------
		do MeanPostRAOutcomes_programs
		//-----------------------------------------------------------------------
		// List of outcome variables described in the Table
		//-----------------------------------------------------------------------		
		global tabvars "ernqa`fq'`lq' anyernqa`fq'`lq'  fqebelow3FPL fqonAFDC avgernqifon fqonAFDCzeroern"
		global nvp = 0
		foreach var in $tabvars {
			global nvp = $nvp+1		
		}	
		
		
		
			
		
		
		
		
	//------------------------------------------------ PART 2  ------------------------------------------------------
	// Bring in Policy Parameters
	//---------------------------------------------------------------------------------------------------------------
	qui use "`rules_dtain'", clear
	keep F3 year size 
	rename F3 F3_
	reshape wide  F3, i(year) j(size)
	sort year 
	qui save "$ddJF/JF_ADFCrules_temp.dta", replace
	
	
	

	
	
	//------------------------------------------------ PART 3  ------------------------------------------------------
	// Intermediate data set used below
	//---------------------------------------------------------------------------------------------------------------	
	tempfile file1
	qui use "`jf_dtain'" if quarter>=`fq'&quarter<=`lq'&kidcount~=., clear
	//---------------------------------------------------------------------------------------------- 
	// AU size which is needed in order to associate the correct FPL 
	// Implicitly we use whichFPL = "nextsizeup"
	//--------------------------------------------------------------------------------------	
	qui gen size = kidcount +1 if kidcount==1|kidcount==2|kidcount==3
	qui replace size = 2 if kidcount==0
	//---------------------------------------------------
	// Merge in policy parameters
	//---------------------------------------------------
	sort year
	qui merge m:1 year using  "$ddJF/JF_ADFCrules_temp.dta"
	qui drop if _m~=3 // drop years for which you do not have the rules
	drop _m	
	//-------------------------------------------------------------------
	// Create vars that identify the earning subgroups 
	//-------------------------------------------------------------------
	create_earning_subgroup 
	// ---------------------------------------------------------------
	// Creates the following variables:
	//	ernqa17			= earning amount in quarter
	//	anyernqa17		= indicator for whether earnings in quarter is positive 
	//	fqebelow1FPL	= indicator for whether earnings in quarter is below 1*FPL
	//	fqebelow3FPL	= indicator for whether earnings in quarter is below 3*FPL
	//	fqonAFDC		= indicator for whether on welfare assistance in quarter
	//	fqonAFDCzeroern = indicator for whether with earnings=0 and on welfare assistance (at least 1 month out of the quarter)
	//----------------------------------------------------------------
	create_vars_in_table `fq' `lq' 
	//----------------------------------------------------------------------
	// Get rid of the long dimension of the data and just stay with wide-format.
	// The wide format is useful to run the propensity score logit models.
	//----------------------------------------------------------------------
	qui gen nm = .
	qui bys id quarter: replace nm = _n
	qui keep if nm==1&quarter==1 //wlog
	drop nm	
	qui save `file1', replace
	capture log close
	log using "`logfileout'", replace	
	
	
	

	
	
	//------------------------------------------------ PART 4  ------------------------------------------------------
	// Mean impacts
	//---------------------------------------------------------------------------------------------------------------
	if `ind_descst'==1 {
	
		//--------------------------------------------------------------------
		// Run pscore logit, predict propensity score (pscore), and generate IPW (pscorewt)
		// for each subgroup based on earnings in 7th Q prior to RA
		//--------------------------------------------------------------------
		qui use `file1', clear
		drop pscorewt
		qui logit e ${pscorevars} , asis
		qui predict double pscore                                      
		gen double pscorewt = e/pscore + (1-e)/(1-pscore) 
		drop pscore	
		foreach g of numlist  0 1 {
			qui count if earning_subgroup==`g'
			qui logit e ${pscorevars_g`g'} if earning_subgroup==`g', asis
			qui predict double pscore                                      
			gen double pscorewt_e`g' = e/pscore + (1-e)/(1-pscore) if earning_subgroup==`g'
			drop pscore
		}
		//-------------------------------------------------------------------------------------------- 
		// Mean outcomes post RA, Overall Sample
		//
		// Note: we first creates the following outcome variables:
		//	avgernqifon			= earning amount in quarter if on welfare assistance 
		//  at least one month out of the quarter which was not created above b/c 
		//  its construction requires pscorewt
		//--------------------------------------------------------------------------------------------
		create_vars_in_table_needpscore `fq' `lq' pscorewt 
		gettable pscorewt "all"
		//-------------------------------------------------------------------------------------------- 
		// Mean outcomes post RA by preRA earning subgroup
		//--------------------------------------------------------------------------------------------
		foreach g of numlist  0 1 {
			preserve
				qui keep if earning_subgroup==`g'
				create_vars_in_table_needpscore `fq' `lq' pscorewt_e`g'
				gettable pscorewt_e`g' `g'
			restore
		}
		//--------------------------------------------------------------------------------------------  
		// Write to screen means of outcome variables
		//--------------------------------------------------------------------------------------------	
		di ""
		di ""
		di ""
		di "================== `whatwedo' (Overall Sample) =================================="
		di " The adjusted means and diff.s in means are:"
		matrix list m_all, noheader format(%6.3f)
		di ""
		di "================== `whatwedo' (Earning Subsample 0) ============================="
		di " The adjusted means and diff.s in means are:"
		matrix list m_g0, noheader format(%6.3f)
		di ""
		di "================== `whatwedo' (Earning Subsample 1) ============================="
		di " The adjusted means and diff.s in means are:"
		matrix list m_g1, noheader format(%6.3f)
	}
		
	
		
		
		
	//------------------------------------------------ PART 5  ------------------------------------------------------
	// Compute SEs by boostrapping.
	//---------------------------------------------------------------------------------------------------------------			
	if `ind_bootst'==1 {
		set seed 112233
		
		//------------------------------------------------------------------------
		// Entire Sample
		//------------------------------------------------------------------------
			di ""
			di ""
			di ""
			di "================== Bootstrapping SEs (Overall Sample) ============================="	
			
			qui use "`file1'" , clear 
			drop pufid
			drop pscorewt*
			rename id pufid

			forvalues r=1(1)`nboots' {
				//--------------------------------------------------
				forvalues i=0(1)10 {
					if `r'==`i'*100 {
						di "- working on bootstrapped sample `r' "
					}
				}
				if `r'==1 {
					di "- working on bootstrapped sample `r' (boostrapping takes time ...)"
				}

				preserve
					//--------------------------------------------------
					// Draw r-th random sample.
					//--------------------------------------------------	
					qui bsample, cluster(pufid) idcluster(id)
					//---------------------------------------------------
					//Now executing code necessary for pscore model and inverse 
					// propensity score weighting (IPW) stuff 
					//---------------------------------------------------
					qui logit e $pscorevars, asis
					qui predict double pscore                                      
					qui gen double pscorewt = e/pscore + (1-e)/(1-pscore)
					//---------------------------------------------------
					// Create using pscore weights the var avgernqifon
					//---------------------------------------------------
					create_vars_in_table_needpscore `fq' `lq' pscorewt 
					//---------------------------------------------------
					// Estimate differences in means
					//---------------------------------------------------
					if `r'==1 {
						qui meandifs $tabvars [aw=pscorewt], by(e) 
						matrix define TabA = r(table)
						forvalues j=1(1)3 {
							local l = (`j'-1)*$nvp+1
							local u = `j'*$nvp	
							matrix define TabA`j' = TabA[1,`l'..`u']
						}
						local stored_names :  colnames TabA3
					}
					else {
						qui meandifs $tabvars [aw=pscorewt], by(e) 
						matrix define TabB = r(table)
						forvalues j=1(1)3 {
							local l = (`j'-1)*$nvp+1
							local u = `j'*$nvp	
							matrix define TabA`j' = TabA`j'\TabB[1,`l'..`u']	
						}
					}
				restore
			} // loop over bs

			//----------------------------------------------------------------------------------------------
			// Use the differences in means for each bootstrapped sample to compute SEs 
			// of the difference in means for each variable
			//----------------------------------------------------------------------------------------------	
			clear
			matrix define ser = J($nvp,3,.)			
			forvalues j=1(1)3 {
				qui svmat TabA`j', names(eqcol)
				capture rename t* * 
				capture rename c* *
				capture rename d* *
				local nv = 0
				foreach var of varlist $tabvars {
					local nv = `nv'+1
					qui sum `var' 
					matrix define ser[`nv',`j'] = r(sd)		
				}	
				clear
			}	
			//----------------------------
			matrix colnames ser = Tmean_SE Cmean_SE ADiff_SE
			di ""
			di ""
			di "================== `whatwedo' (Overall Sample) ===================================="
			if `ind_descst'==1 {
				matrix define `whatwedo' = [m_all,ser]
				di " The bootstrapped SEs for the adjusted diff.s in means appear in the last 3 cols:"
				matrix list `whatwedo', noheader format(%6.3f)
			}
			else {
				matrix rownames ser = `stored_names'
				di ""
				di " The bootstrapped SEs for the adjusted diff.s in means are:"
				matrix list ser, noheader format(%6.3f)
			}
		
		
		//------------------------------------------------------------------------------------------
		// preRA earning groups
		//------------------------------------------------------------------------------------------		
		foreach g of numlist  0 1 {
			di ""
			di ""
			di ""
			di "================== Bootstrapping SEs (Earning Subsample `g') ======================"
			qui use `file1' if earning_subgroup==`g', clear
			drop pufid
			drop pscorewt*
			rename id pufid
			
			forvalues r=1(1)`nboots' {
				//--------------------------------------------------
				forvalues i=0(1)10 {
					if `r'==`i'*100 {
						di "- working on bootstrapped sample `r' "
					}
				}
				if `r'==1 {
					di "- working on bootstrapped sample `r' (boostrapping takes time ...)"
				}

				preserve
					//---------------------------------------------------
					// Draw a random sample from existing data
					//---------------------------------------------------		
					qui bsample, cluster(pufid) idcluster(id)
					//---------------------------------------------------
					//Now executing code necessary for pscore model and inverse propensity score weighting (IPW) stuff 
					//---------------------------------------------------
					qui logit e ${pscorevars_g`g'}, asis
					qui predict double pscore                                      
					qui gen double pscorewt = e/pscore + (1-e)/(1-pscore)
					//---------------------------------------------------
					// Create using pscore weights the var avgernqifon
					//---------------------------------------------------
					create_vars_in_table_needpscore `fq' `lq' pscorewt 					
					//---------------------------------------------------
					// Record averages and their difference
					//---------------------------------------------------
					if `r'==1 {
						qui meandifs $tabvars [aw=pscorewt], by(e) 
						matrix define TabA = r(table)
						forvalues j=1(1)3 {
							local l = (`j'-1)*$nvp+1
							local u = `j'*$nvp	
							matrix define TabA`j' = TabA[1,`l'..`u']
						}
						local stored_names :  colnames TabA3
					}
					else {
						qui meandifs $tabvars [aw=pscorewt], by(e) 
						matrix define TabB = r(table)
						forvalues j=1(1)3 {
							local l = (`j'-1)*$nvp+1
							local u = `j'*$nvp	
							matrix define TabA`j' = TabA`j'\TabB[1,`l'..`u']	
						}
					}
				restore
			} // loop over bs
			//----------------------------------------------------------------------------------------------
			// Use the differences in means for each bootstrapped sample 
			// to compute SEs of the difference in the proportions  between control and treatment group
			//----------------------------------------------------------------------------------------------
			clear
			matrix define ser = J($nvp,3,.)			
			forvalues j=1(1)3 {
				qui svmat TabA`j', names(eqcol)
				capture rename t* * 
				capture rename c* *
				capture rename d* *
				local nv = 0
				foreach var of varlist $tabvars {
					local nv = `nv'+1
					qui sum `var' 
					matrix define ser[`nv',`j'] = r(sd)		
				}	
				clear
			}	
			matrix colnames ser = Tmean_SE Cmean_SE ADiff_SE
			di ""
			di "================== `whatwedo' (Earning Subsample `g') ============================="
			if `ind_descst'==1 {
				matrix define `whatwedo' = [m_g`g',ser]
				di " The bootstrapped SEs for the adjusted diff.s in means appear in the last 3 cols:"
				matrix list `whatwedo',  noheader format(%6.3f)
			}
			else {
				matrix rownames ser = `stored_names'
				di " The bootstrapped SEs for the adjusted diff.s in means are:"
				matrix list ser	, noheader format(%6.3f)
			}
		} // loop over preRA earning groups
	} // if you want to boostrap SE
	capture log close
	erase "$ddJF/JF_ADFCrules_temp.dta"
end // encapsulating program





	//------------------------------------------------ PART 6  ------------------------------------------------------
	// Execution
	//---------------------------------------------------------------------------------------------------------------
	MeanPostRAOutcomes "`jf_dtain'" "`rules_dtain'"
	
	
	
	


